#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#define OPENERR 1
#define BADDATA 2
#define NUM_ROWS 9
#define NUM_COLUMNS 8

#define DEBUG
enum {FAILURE, SUCCESS};

static char puzzle[NUM_ROWS][NUM_COLUMNS+1];
/**
@fn writeMessageandExit(const char *message)
writes a message and exits program
@param message message to be printed on stderr
@param exit code for the program
*/
void writeMessageandExit(const char *message, int exitCode)
{
    fprintf(stderr, "%s\n", message);
    exit(exitCode);
}
/**
@fn openAndCheck(const char *filePath, const char *mode)
Attempts to open a file given by its path, used in mode given by mode.
If the operation is unsuccessful exits program with code
@param filePath path to the file to be opened
@param mode mode of opening: read  or write
@return pointer to a FILE structure
*/
FILE *openAndCheck(const char *filePath, const char *mode)
{
    FILE *f = fopen(filePath, mode);
    if (NULL == f)
    {
        perror(filePath);
        exit(OPENERR);
    }
    return f;
}
/**
fn readDataLine(FILE *f, char *string, int *pRow, int *pColumn, char *pDirection)
reads data for one string toplace in the puzzle
@param string string toplace in puzzle
@param pRow pointer to where to store start row of string in the puzzle
@param pDirection pointer where to store direction of placement
@param pComlumn pointer to where to store start column of string in the puzzle
@return SUCCESS if read is successful or FAILURE otherwise
*/
int readDataLine(FILE *f, char *string, int *pRow, int *pColumn, char *pDirection)
{
    char buffer[20];
    if (NULL == fgets(buffer, sizeof(buffer), f))
    {
#ifdef DEBUG
            printf("%s %d: FAILURE\n", __FUNCTION__, __LINE__);
#endif // DEBUG
        return FAILURE;
    }
    sscanf(buffer, "%s%d%d %c", string, pRow, pColumn, pDirection);
#ifdef DEBUG
    // print what was read
    printf("%s: \"%s\" %d %d %c\n", __FUNCTION__, string, *pRow, *pColumn, *pDirection);
    printf("%s %d: SUCCESS\n", __FUNCTION__, __LINE__);
#endif // DEBUG
    return SUCCESS;
}
/**
@fn placeString(const char *string, int row, int col, char dir)
places a string at a given start (row, column) position in the given direction
@param string string to place in puzzle
@param row start row
@param column start column
@param direction one of 'H' for horizontal or 'V' for vertical
@return SUCCESS if string fits or FAILURE if TRUNCATED
*/
int placeString(const char *string, int row, int column, char direction)
{
#ifdef DEBUG
    // print function arguments
    printf("%s: \"%s\" %d %d %c\n", __FUNCTION__, string, row, column, direction);
#endif // DEBUG
    // adjust indices to run from 0 not 1 as in input
    --row;
    --column;
    int j =0;
    switch(direction)
    {
    case 'H':
        if (row >= NUM_ROWS)
            writeMessageandExit("Row too big", BADDATA);
        for (int i = column; i < NUM_COLUMNS && string[j]; i++, j++)
            puzzle[row][i] = puzzle[i][column] == string[j]? toupper(string[j]):string[j];
        if (string[j])
        {
#ifdef DEBUG
            printf("%s %d: FAILURE\n", __FUNCTION__, __LINE__);
#endif // DEBUG
            return FAILURE;
        }
        break;
    case 'V':
        if (column >= NUM_COLUMNS)
            writeMessageandExit("Column too big", BADDATA);
        for (int i=row; i < NUM_ROWS && string[j]; i++, j++)
            puzzle[i][column] = puzzle[i][column] == string[j]? toupper(string[j]):string[j];
        if (string[j])
        {
#ifdef DEBUG
            printf("%s %d: FAILURE\n", __FUNCTION__, __LINE__);
#endif // DEBUG
            return FAILURE;
        }
        break;
    default:
        writeMessageandExit("Illegal direction value", BADDATA);
#ifdef DEBUG
            printf("%s %d: FAILURE\n", __FUNCTION__, __LINE__);
#endif // DEBUG
        return FAILURE;
        break;
    }
#ifdef DEBUG
            printf("%s %d: SUCCESS\n", __FUNCTION__, __LINE__);
#endif // DEBUG
    return SUCCESS;
}
int main()
{
    char string[20];
    int row, col;
    char direction;
    // init puzzle
    for (int i=0; i < NUM_ROWS; i++)
        for (int j=0; j < NUM_COLUMNS; j++)
            puzzle[i][j] = '*';
    // open input file
    FILE *fpIn=openAndCheck("words.txt", "r");
    // for each line of input
    for (;;)
    {
        //  read line data
        if (readDataLine(fpIn, string, &row, &col, &direction) == SUCCESS)
        {
            placeString(string, row, col, direction);
        }
        else break;
    }
    // close input file
    fclose(fpIn);
    // open output file
    FILE *fpOut=openAndCheck("puzzle.txt", "w");
    // for each row of the puzzle
    for (int i =0; i < NUM_ROWS; i++)
    //  write a line i the output
        fprintf(fpOut, "%s\n", puzzle[i]);
    // close output file
    fclose(fpOut);
    return 0;
}
